/*
	lkmonitor (Linux Kernel Monitor)

	Application for monitoring and tuning a Linux kernel.

	Copyright (C) 2005-2008  Fernando Apesteguía

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "lkmonitor.h"
#include "lininfo.h"
#include "others.h"
#include "update.h"
#include "procupdate.h"


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_widget_show(create_about());
}


void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  lkmonitor_exit();
  gtk_main_quit();
  
}

gboolean
on_window1_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	extern gboolean is_icon;
  	
	is_icon=TRUE;
	gtk_widget_hide(widget);
	return TRUE; //With TRUE the signal is not propagated up.  
	
}

void
on_btn_load_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	update_settings();

}

void
on_btn_write_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  lkmonitor_write_settings();
  //Now, we should read settings again to properly update the data.
update_settings();
}

void
on_btn_dev_load_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  update_devs();
}

void
on_btn_comm_reload_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	update_channels();
}


void
on_gtv_processes_row_activated         (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{

  GtkWindow *window;
  GtkTreeModel *model;
  GtkTreeIter iter;
  int pid;
  gchar *name=NULL;

  window=GTK_WINDOW(create_window2());

  model=GTK_TREE_MODEL(gtk_tree_view_get_model(treeview));
  if(gtk_tree_model_get_iter(model,&iter,path)){
	  gtk_tree_model_get(model,&iter,0,&pid,1,&name,-1);

	  /* Call the function to collect the process information */
		update_process_window(pid,name,window);

  	gtk_widget_show(GTK_WIDGET(window));
  }

}



void
on_tv_maps_row_activated               (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
	GtkWindow *window;
	GtkTreeModel *model;
	GtkTreeIter iter;
	GString *title=g_string_new("");
	gchar *address;
	char *p;
	int pid;

	/* Get the address interval */
	model=GTK_TREE_MODEL(gtk_tree_view_get_model(treeview));
	if(gtk_tree_model_get_iter(model,&iter,path)){
		gtk_tree_model_get(model,&iter,0,&address,-1);

		/* Get the PID from the window title... 
		* This is a bit dirty, I guess we can do it
	 	* in a better way...
	 	* */
		window=GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(treeview)));
		g_string_printf(title,"%s",gtk_window_get_title(window));

		if((p=strstr(title->str, "PID"))!=NULL){
			sscanf(p,"PID: %d",&pid);
			/* Get details for the selected memory region */
			update_mm_details(address,window,pid);
		}
	}
	/* Free the GString */
	g_string_free(title,TRUE);
}


void
on_modules_refresh_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{

	update_modules_info();

}

