
/*
	lkmonitor (Linux Kernel Monitor)

	Application for monitoring and tuning a Linux kernel.

	Copyright (C) 2005-2008  Fernando Apesteguía

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <glib.h>
#include <gtk/gtk.h>

#include "eggtrayicon.h"
#include "support.h"

#include "others.h"
#include "lininfo.h"



/*! \struct docklet lkmonitor.c
*   \brief Represents the panel icon.
*  This struct contains several fields that represent the icon showed in
*  the gnome panel 
*/
struct {
	EggTrayIcon *eti;
    GtkWidget *eb;
    GtkWidget *image;	  
  }	docklet;


extern GThread *gt_collector;
extern GtkWidget *window1;

/*! Stops collector thread and runs outside event loop.
*/

void lkmonitor_exit()
{
  	extern gboolean collector_exit;

	collector_exit=TRUE;
	gtk_main_quit();
}

/*! Shows the window when click on the docklet.
* \param w the clicked widget
*/
void docklet_onclick(GtkWidget *w)
{
	extern gboolean is_icon;
	
	gtk_widget_show(window1);
	is_icon=FALSE;
	
}

/*! Creates the tray_icon.
* Based on the code of jabber IM client.
*/
void lkmonitor_create_tray_icon(){

  gint w = 0;
  gint h = 0;
	
  docklet.eti = egg_tray_icon_new("lkmonitor");
  docklet.eb = gtk_event_box_new();
  docklet.image = gtk_image_new();

  GdkPixbuf *pb = NULL;
  GdkPixbuf *pb_scaled = NULL;

  pb =create_pixbuf("spider-shadow32.png");
  gtk_icon_size_lookup(GTK_ICON_SIZE_LARGE_TOOLBAR, &w, &h);
  pb_scaled = gdk_pixbuf_scale_simple(pb, w, h, GDK_INTERP_BILINEAR);
  gtk_image_set_from_pixbuf(GTK_IMAGE(docklet.image), pb_scaled);


  gtk_container_add(GTK_CONTAINER(docklet.eb), docklet.image);
  gtk_container_add(GTK_CONTAINER(docklet.eti), docklet.eb);
  gtk_widget_show_all(GTK_WIDGET(docklet.eti));
  
 
  g_signal_connect(G_OBJECT(docklet.eb), "button-press-event", G_CALLBACK(docklet_onclick), window1);
  
  
  g_object_ref(G_OBJECT(docklet.eti));
}

/*! Searchs for net devices and place them in the combo box
 *\param w Main window
 *\return 0 on success, -1 otherwise
*/
int lkmonitor_load_net_dev(GtkWidget *w)
{
  char *contents,*contentsp,*p;
  char dev_name[10];
  int i=0;
  GtkComboBox *gcb_netdevs;

  if(g_file_get_contents(NET_DEVS,&contents,NULL,NULL)==FALSE){
	g_warning("No net devices file");
  	gcb_netdevs=GTK_COMBO_BOX(lookup_widget(w,"cb_net_devs"));
	gtk_widget_set_sensitive(GTK_WIDGET(gcb_netdevs),FALSE);
	g_free(contents);
	return -1;
  }

  gcb_netdevs=GTK_COMBO_BOX(lookup_widget(w,"cb_net_devs"));

  /* File exists */

  contentsp=skip_lines(contents,2);
  
  while(sscanf(contentsp,"%10s:",dev_name)!=EOF){
	/* Remove the ':' */
	if((p=strchr(dev_name,':'))!=NULL)
		*p='\0';
	gtk_combo_box_insert_text(gcb_netdevs,i++,dev_name);
	contentsp=skip_lines(contentsp,1);
  }
  
  gtk_combo_box_set_active(gcb_netdevs,0);

	/* Clean up */
  	g_free(contents);
  	return 0;

}

/*! Initialize additional widgets.
* This function initialize more widgets like GtkTreeViews.
* \param window main window
*/
void lkmonitor_init_widgets(GtkWidget *window)
{

  GtkListStore *block_list;
  GtkListStore *char_list;
  GtkListStore *process_list;
  GtkListStore *modules_list;
  GtkCellRenderer *b_renderer;
  GtkCellRenderer *c_renderer;
  GtkCellRenderer *p_renderer;
  GtkTreeView *gtv_char;
  GtkTreeView *gtv_block;
  GtkTreeView *gtv_processes;
  GtkTreeView *gtv_modules;
  
  /* For the gtktextviews */
  GtkTextView *gtv;
  GtkTextBuffer *gtktextb_ioports;
  GtkTextBuffer *gtktextb_iomem;
  GtkTextBuffer *gtktextb_dma;
  GtkTextBuffer *gtktextb_fs;
  GtkTextBuffer *gtktextb_pfs;
  

	/*Tree views to show devices*/
	/*For block devices*/
	
	/*First column*/
  gtv_block= (GtkTreeView *)lookup_widget(window,"gtv_block");
  block_list=gtk_list_store_new(2,G_TYPE_INT, G_TYPE_STRING);
  gtk_tree_view_set_model(GTK_TREE_VIEW(gtv_block),GTK_TREE_MODEL(block_list));
 
  b_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_block),
                                               -1,      
                                               _("Number"),  
                                               b_renderer,
                                               "text", 0,
                                               NULL);

  /*Second column*/

  b_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_block),
                                               -1,      
                                               _("Device"),  
                                               b_renderer,
                                               "text", 1,
                                               NULL);
											   
  
  /*For char devices*/	
  
  /*First column*/
  gtv_char= (GtkTreeView *)lookup_widget(window,"gtv_char");
  char_list=gtk_list_store_new(2,G_TYPE_INT, G_TYPE_STRING);
  gtk_tree_view_set_model(GTK_TREE_VIEW(gtv_char),GTK_TREE_MODEL(char_list));
 
  c_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_char),
                                               -1,      
                                               _("Number"),  
                                               c_renderer,
                                               "text", 0,
                                               NULL);

  /*Second column*/

  c_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_char),
                                               -1,      
                                               _("Device"),  
                                               c_renderer,
                                               "text", 1,
                                               NULL);
  
  /*For Processes List*/	
  
  /*First column*/
  gtv_processes= (GtkTreeView *)lookup_widget(window,"gtv_processes");
  process_list=gtk_list_store_new(2,G_TYPE_INT, G_TYPE_STRING);
  gtk_tree_view_set_model(GTK_TREE_VIEW(gtv_processes),GTK_TREE_MODEL(process_list));
 
  p_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_processes),
                                               -1,      
                                               _("PID"),  
                                               p_renderer,
                                               "text", 0,
                                               NULL);

  /*Second column*/

  c_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_processes),
                                               -1,      
                                               _("Command"),  
                                               c_renderer,
                                               "text", 1,
                                               NULL); 

  /* For Modules list */  
                                             
  gtv_modules= (GtkTreeView *)lookup_widget(window,"gtv_modules");
  modules_list=gtk_list_store_new(5,G_TYPE_STRING, G_TYPE_STRING,G_TYPE_STRING,
					G_TYPE_STRING,G_TYPE_STRING);
  
  gtk_tree_view_set_model(GTK_TREE_VIEW(gtv_modules),GTK_TREE_MODEL(modules_list));
 
  /*First column*/
  p_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_modules),
                                               -1,      
                                               _("Name"),  
                                               p_renderer,
                                               "text", 0,
                                               NULL);

  /*Second column*/

  c_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_modules),
                                               -1,      
                                               _("Size"),  
                                               c_renderer,
                                               "text", 1,
                                               NULL); 

  /*Third column*/
  p_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_modules),
                                               -1,      
                                               _("RefCount"),  
                                               p_renderer,
                                               "text", 2,
                                               NULL);

  /*Forth column*/

  c_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_modules),
                                               -1,      
                                               _("State"),  
                                               c_renderer,
                                               "text", 3,
                                               NULL); 

  /*Fifth column*/

  c_renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (gtv_modules),
                                               -1,      
                                               _("Address"),  
                                               c_renderer,
                                               "text", 4,
                                               NULL); 

/* We need to initialize the gtktextviews adding buffers to them */

/* First, create the GtkTextBuffers */

gtktextb_ioports=gtk_text_buffer_new(NULL);
gtktextb_iomem=gtk_text_buffer_new(NULL);
gtktextb_dma=gtk_text_buffer_new(NULL);
gtktextb_fs=gtk_text_buffer_new(NULL);
gtktextb_pfs=gtk_text_buffer_new(NULL);

/* Attach them */
gtv=GTK_TEXT_VIEW(lookup_widget(window1,"gtextv_ioports"));
gtk_text_view_set_buffer(gtv,gtktextb_ioports);

gtv=GTK_TEXT_VIEW(lookup_widget(window1,"gtextv_iomem"));
gtk_text_view_set_buffer(gtv,gtktextb_iomem);

gtv=GTK_TEXT_VIEW(lookup_widget(window1,"gtextv_dma"));
gtk_text_view_set_buffer(gtv,gtktextb_dma);

gtv=GTK_TEXT_VIEW(lookup_widget(window1,"gtextv_fs"));
gtk_text_view_set_buffer(gtv,gtktextb_fs);

gtv=GTK_TEXT_VIEW(lookup_widget(window1,"gtextv_pfs"));
gtk_text_view_set_buffer(gtv,gtktextb_pfs);


/* Search for net devices and fill the combo box */
 lkmonitor_load_net_dev(window1);

}


/*! Fills devices treeviews with data.
* \param model the model
* \param gtv the treview to fill
* \param iter iterator for the model
* \param buffer string with data to be inserted into the model
* \param type number identifying the type of treeview to fill
*/

void lkmonitor_fill_tv(GtkListStore *model, GtkTreeView *gtv,
					GtkTreeIter iter, char *buffer, int type)
{
	g_assert(model!=NULL && gtv!=NULL && buffer !=NULL);
	
	char *pointer;
	//Now add data to model

	pointer=buffer;

  if(type==TV_DEVS){
	
	char name[50];
	int num;

	while(*pointer!='\0'){
		gtk_list_store_append(model,&iter);
		sscanf(pointer,"%d %s",&num,name);
		gtk_list_store_set(model,&iter,0,num,1,name,-1);
		pointer=strchr(pointer,'\n')+1;
	}
  }

  if(type==TV_MODS){
	
	char name[20], size[6], refcount[3], state[10], address[20];	

	while(*pointer!='\0'){
		gtk_list_store_append(model,&iter);
		sscanf(pointer,"%s %s %s %s %s",name,size,refcount,state,address);
		gtk_list_store_set(model,&iter,0,name,1,size,2,refcount,3,state,4,address,-1);
		pointer=strchr(pointer, '\n')+1;
	}
  }
}

/*! Fills devices treeviews with data.
* \param model the model
* \param gtv the treview to fill
* \param iter iterator for the model
* \param buffer string with data to be inserted into the model
*/

void lkmonitor_fill_tv_modules(GtkListStore *model, GtkTreeView *gtv,
					GtkTreeIter iter, char *buffer)
{
	g_assert(model!=NULL && gtv!=NULL && buffer !=NULL);
	
	char *pointer;
	char name[50];
	int num;
	//Now add data to model

	pointer=buffer;

	while(*pointer!='\0'){
		gtk_list_store_append(model,&iter);
		sscanf(pointer,"%d %s",&num,name);
		gtk_list_store_set(model,&iter,0,num,1,name,-1);
		pointer=strchr(pointer,'\n')+1;
	}
}

/*! Write new settings.
*/
void lkmonitor_write_settings()
{
  //This is called from a callback, so we are still running in a gtk lock.
  GtkEntry *ent;
  GtkLabel *lbl;
  GtkSpinButton *spb;
  GString *gs;
  int val1,val2,val3,val4;
	
/*For Identification*/
  ent=GTK_ENTRY(lookup_widget(window1,"ent_hostname"));
  gs=g_string_new(gtk_entry_get_text(ent));
  write_string_to_file(gs->str,HOST_NAME);
	
  ent=GTK_ENTRY(lookup_widget(window1,"ent_domainname"));
  gs=g_string_new(gtk_entry_get_text(ent));
  write_string_to_file(gs->str,DOMAIN_NAME);
	
/*For Files*/
  	
  spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_fdmax"));
  val1=(int)gtk_spin_button_get_value_as_int(spb);

  write_values_to_file(3,FILE_MAX,val1);

/*For Inodes*/
	lbl=GTK_LABEL(lookup_widget(window1,"lbl_inmax"));
	val1=atoi((char *)gtk_label_get_text(lbl));
	
	lbl=GTK_LABEL(lookup_widget(window1,"lbl_ina"));
	val2=atoi((char *)gtk_label_get_text(lbl));
  	
	write_values_to_file(2,INODES,val1,val2);

/*For POSIX signals*/
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_mq"));
	val1=gtk_spin_button_get_value_as_int(spb);
	write_values_to_file(1,P_MAX_Q,val1);
	
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_max_mpq"));
	val1=gtk_spin_button_get_value_as_int(spb);
	write_values_to_file(1,P_MAX_M,val1);
	
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_max_ms"));
	val1=gtk_spin_button_get_value_as_int(spb);
	write_values_to_file(1,P_MAX_S,val1);

/*For shared memory*/
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_max_pages"));
	val1=gtk_spin_button_get_value_as_int(spb);
	write_values_to_file(1,PAGE_MAX,val1);
	
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_max_segs"));
	val1=gtk_spin_button_get_value_as_int(spb);
	write_values_to_file(1,SEG_MAX,val1);
	
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_seg_size"));
	val1=gtk_spin_button_get_value_as_int(spb);
	write_values_to_file(1,SEG_SIZE,val1);
	
/*For semaphores*/
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_sem_per_id"));
	val1=gtk_spin_button_get_value_as_int(spb);
	
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_sem_val"));
	val2=gtk_spin_button_get_value_as_int(spb);
	
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_ops"));
	val3=gtk_spin_button_get_value_as_int(spb);
	
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_sem_id"));
	val4=gtk_spin_button_get_value_as_int(spb);
	
	write_values_to_file(4,SEM,val1,val2,val3,val4);
	
/*For threads*/
	spb=GTK_SPIN_BUTTON(lookup_widget(window1,"spb_maxthreads"));
	val1=gtk_spin_button_get_value_as_int(spb);
	
	write_values_to_file(1,THREADS_MAX,val1);
}



/*! Converts an int to a string
 *\param num Integer 
 *\return char* with the ASCII representation of num
*/
char *lkmonitor_itoa(int num)
{
  GString *gs=g_string_new("");

  g_string_printf(gs,"%d",num);

  return gs->str;

}
