
/*
	lkmonitor (Linux Kernel Monitor)

	Application for monitoring and tuning a Linux kernel.

	Copyright (C) 2005-2008  Fernando Apesteguía

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <sys/types.h>
#include <signal.h>

#include "interface.h"
#include "support.h"
#include "lininfo.h"
#include "update.h"
#include "lkmonitor.h"

#include <unistd.h>
#include <stdio.h>
#include <glib.h>


GtkWidget *window1;



/*Struct definitions*/

struct cpu_info cpu;
struct mem_info mem;
struct dev_info devs;
struct settings_info settings;
struct stats_info stats;
struct acpi_info acpi;
struct cmd_ver cmdversion;
struct channels_info cn_info;
struct net_dev_info netinfo;
struct fs_info	fsinfo;
struct modules_info modinfo;

pthread_t thread_id;
GThread *gt_collector=NULL;

gboolean is_icon=FALSE;
gboolean collector_exit=FALSE;

/*! Function that represent the collector thread behaviour
* Collector thread runs into an infinite loop, scanning window
* and refreshing information if required.
* \param nothing Required for POSIX compatibility. Not used
*/
void
refresh (void* nothing)
{
  int page;
  GtkNotebook *nb;

 nb = (GtkNotebook *) (lookup_widget (window1, "nb_main"));
  //Get current page from the notebook
 while(!collector_exit){
 if(!is_icon){	
	 page = (int) gtk_notebook_get_current_page (nb);
  switch (page)
    {
	case 0:
		update_cpu_mhz();
		break;
    case 1:
      update_mem ();
      break;
    case 2:
	 // update_devs();
      break;
    case 3:
	//	update_settings();
		break;
	case 4:
	  	update_misc();
	  	break;
	case 5:
		update_process();
		break;
	case 6:
		break;
	case 7:
		update_net_info();
		break;
	case 8:
  		update_fs_info();
		break;
    }
 }
	sleep(1);

}
}

/*! Program entry point
* \param argc Number of arguments
* \param argv string array (for arguments)
* \return 0 if normal program exit
*/
int
main (int argc, char *argv[])
{
	gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
                      argc, argv,
                      GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR,
                      NULL);
	
#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif


  gdk_threads_init ();
	
	
  gtk_set_locale ();
  gtk_init (&argc, &argv);


  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any crefresh);omponents that you don't want shown initially.
   */
  lkmonitor_create_tray_icon();
  window1 = create_window1 ();
  /*Set some widget properties*/
  
  lkmonitor_init_widgets(window1);
  
  gtk_widget_show (window1);
  
  

  /*First calls for static information*/
 
 update_cpu ();
  update_devs();
  update_settings();
  update_cmd_ver();
  update_channels();
  update_net_info();
  update_modules_info();
  
  if(get_battery_info(&acpi.battery,BATT_INFO)==-1)
	  get_battery_info(&acpi.battery,BATT_INFO1);
  

gdk_threads_enter ();  //Begin of critial section

/*Launch thread*/

gt_collector=g_thread_create((GThreadFunc)refresh,NULL,TRUE,NULL);
if(gt_collector!=NULL)
	g_message("Collector thread created");
else
	/*Error creating thread*/
	g_critical("Could not create thread!");


gtk_main ();

gdk_threads_leave(); //Marks end of critical section

g_thread_join(gt_collector); /*Wait for collector thread to finish*/

return 0;
}
