
/*
	lkmonitor (Linux Kernel Monitor)

	Application for monitoring and tuning a Linux kernel.

	Copyright (C) 2005-2008  Fernando Apesteguía

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <ctype.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include "others.h"


/*!Filter for scandir. It passes only dir names which represent a PID
*\param entry Pointer to dirent struct to traverse the list of files
*\return 0 if the file doesn't match or 1 if it matches
*/
static int
filter (const struct dirent *entry)
{
  char ext;			

  if ((strcmp (entry->d_name, ".") == 0) ||
      (strcmp (entry->d_name, "..") == 0))
    return (0);	/*We're not interested in '.' or '..' directories*/

  ext=entry->d_name[0];

  if (isdigit(ext)){
  	return(1);
  }
  else
    return (0);

}			

/*!Get the list of processes
*\param proclist Triple pointer to dirent struct. Here we write the list of processes.
*\return Number of processes attached or errno if error.
*/
int get_proc_list(struct dirent ***proclist){
	int numfich;
	
	if ((numfich = scandir ("/proc", proclist, &filter, numsort)) == -1)
    {
      perror ("scandir error");
      return (errno);
    }
	
	return(numfich);
}
