
/*
	lkmonitor (Linux Kernel Monitor)

	Application for monitoring and tuning a Linux kernel.

	Copyright (C) 2005-2008  Fernando Apesteguía

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/* Header for procinfo */
#include <gtk/gtk.h>

#define DEF_SZ 15

/*!\struct procinfo procinfo.h
 *\brief Detailed process information
 * This structure is the main structure where we keep the 
 * specific process information.
*/
struct procinfo{
	int pid;						///< PID of the process
	GString *env;				///< Environment
	GString *maps;				///< Memory maps
	GString *cmdline;			///< Command line of the process
	char size[DEF_SZ];		///< Size (memory)
	char locked[DEF_SZ];		///< Locked (memory)
	char rss[DEF_SZ];			///< Resident memory
	char data[DEF_SZ];		///< Data (memory)
	char stack[DEF_SZ];		///< Stack size
	char exe[DEF_SZ];			///< Executable size
	char lib[DEF_SZ];			///< Libraries
	char state[DEF_SZ+5]; 	///< State. Big enough to say 'uninterruptible' :)
	char sleepavg[DEF_SZ];	///< Sleep average
	char threads[DEF_SZ];	///< Number of threads in this process
};

/*!\struct mm_details procinfo.h
 *\brief Memory region details
 * This structure is used to keep details of certain memory regions
*/
struct mm_details{
	char size[DEF_SZ];			///< Size
	char rss[DEF_SZ];				///< Resident
	char sh_clean[DEF_SZ];		///< Shared clean
	char sh_dirty[DEF_SZ];		///< Shared dirty
	char priv_clean[DEF_SZ];	///< Private clean
	char priv_dirty[DEF_SZ];	///< Private dirty
};

int get_process_info(const int pid, struct procinfo *pi,struct dirent ***file_list,int *numfiles); ///< Gets process information
int get_mm_details(const int pid, struct mm_details *details,const char *mmregion); ///< Get memory region details
